from clonekin import request
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from collections import defaultdict


class SnPool():
    def __init__(self, query_list):  # load data
        self.data = []
        self.target = []
        self.initiation_list = []
        self.variation_list = pd.DataFrame()
        self.result = pd.DataFrame()
        for i in query_list:
            self.data.append(i.result)
            self.target.append(i.target)
        self.working()

    def ext_gene(self):
        genelist = []
        for i in self.data:
            genelist.extend(i['gene'].to_list())
        return list(set(genelist))

    def working(self):
        genelist = self.ext_gene()
        data = []
        for i in self.data:
            local_gene = i['gene'].to_list()
            local_var = i['variation_number'].to_list()
            local_tab = dict(zip(local_gene, local_var))
            for j in genelist:
                if not j in local_gene:
                    local_tab[j] = 0
            sum_var = sum(local_tab.values())
            for k, v in local_tab.items():
                local_tab[k] = (v + 0.1) * 100 / sum_var
            data.append(local_tab)
        sum_dict = {}
        for _ in data:
            for k, v in _.items():
                sum_dict.setdefault(k, []).append(v)
        cal_dict = {}
        for k, v in sum_dict.items():
            cal_dict[k] = stat(v)
        self.result = pd.DataFrame.from_dict(cal_dict, orient='index', columns=self.target)
        self.initiation_list = pd.DataFrame.from_dict(sum_dict,orient='index', columns=self.target)


    def plot(self, gene):
        x = self.target
        y = list(self.variation_list.loc[self.variation_list['gene'] == gene].values)[0][1:]
        plt.scatter(x, y)
        plt.title('Frequency of ' + gene)


def stat(list_ele):  # 计算新行
    total = 0
    iter_num = len(list_ele)
    new_list = []
    j = 1
    for i in range(iter_num):
        total = total + list_ele[i]
    for i in range(iter_num):
        if total != 0:
            m = list_ele[i] / total
        else:
            m = 1
        j = j * m
        new_list.append(m)
    alt = []
    for i in list_ele:
        alt.append(i)
    alt.sort()
    sec = alt[-2]
    if max(list_ele) != min(list_ele):
        j = (j / (max(new_list) * (max(list_ele) - sec) ** (len(list_ele) - 1))) ** (1 / (len(list_ele) - 1))
    else:
        j = 'NA'
    m_list = ['_'] * len(new_list)
    if j != 'NA':
        for i in range(len(new_list)):
            if new_list[i] == max(new_list):
                m_list[i] = j / new_list[i]
    return m_list


def ext_gene(datalist):
    genelist = []
    for i in datalist:
        genelist.extend(i['gene'].to_list())
    return list(set(genelist))









import sys
import zlib
from collections import Counter

import numpy as np
import pandas as pd
import json
import requests
from clonekin import snpool


class TcbData():  # 一个query对应一组数据
    def __init__(self, type=None, name=None, num=None, threshold=None):
        if num == None:
            self.num = '100'
        else:
            self.num = str(num)
        if threshold == None:
            self.threshold = 4
        else:
            self.threshold = threshold
        self.type = type
        self.name = name
        if self.type != None and self.name != None:
            self.url = 'http://166.111.153.73/olmb/api/reqscr.php?t=' + self.type + '&c=' + self.name + '&n=' + self.num
            self.get = requests.get(self.url)
            self.data = json.loads(zlib.decompress(self.get.content))
            self.status = self.get.status_code
            self.info = self.get.reason
            self.co_data = pd.DataFrame()
            if self.check():
                self.impact = self.get_impact()
                self.result = self.summary()
            else:
                print('=======================================================\n'
                      'DATA ERROR::There might be something wrong with dataset\n'
                      '=======================================================' + self.type + self.name,
                      file=sys.stderr)
        else:
            print('======================================================================\n'
                  'URL ERROR::Not a valid query, there might be some missing information!\n'
                  '======================================================================', file=sys.stderr)

    def check(self):  # 检查文件是否完整
        if self.status == 200:
            if self.data['length'] != 0:
                return True
        else:
            print(self.info, file=sys.stderr)

    def get_impact(self):  # 得到impact文件
        data = self.data['data']
        co_data = []
        for k, v in data.items():
            for i in v:
                if i[2] != '' and i[1] != 'synonymous_variant':
                    i.append(k)
                    co_data.append(i)
                if i[2] == '' and 'splice' in i[1]:
                    i[2] = 'splice_variant'
                    i.append(k)
                    co_data.append(i)
        return pd.DataFrame(co_data)

    def summary(self):  # 原genetorch filter函数
        conc = self.impact
        filter = conc.groupby([0, 2]).filter(lambda x: len(x) <= self.threshold)
        result = get_re(filter)
        n = result.shape[0]
        result.index = list(range(n))
        return result

def get_re(co_data):
    co_data = co_data.drop_duplicates()
    genelist = list(set(co_data[0].to_list()))
    line = []
    for gene in genelist:
        temp_df = co_data[co_data[0] == gene]
        sample = list(set(temp_df[3].to_list()))
        size = len(sample)
        var = Counter(temp_df[2].to_list())
        var = dict(var)
        var_num = len(var)
        line.append([sample, size, gene, var, var_num])
    result = pd.DataFrame(line, columns=['sample', 'size', 'gene', 'variation', 'variation_number'])
    result = result[~result['variation'].isin([[]])]
    result = result.sort_values(by=['size'], ascending=False)
    result = result.reset_index(drop=True)
    return result



from distutils.core import setup
from setuptools import find_packages

with open("README.md", "r", encoding='utf-8') as f:
    long_description = f.read()

setup(name='tcb_project',  # 包名
      version='0.0.1',  # 版本号
      description='Collecting C.elegans WGS Data with obvious phenotype',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Guo_Zhengyang',
      author_email='guozhengyang980525@yahoo.co.jp',
      install_requires=['pandas'],
      license='MIT License',
      packages=find_packages(),
      platforms=["all"],
      classifiers=['Programming Language :: Python :: 3', 'Development Status :: 4 - Beta', ]
      )